# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from .. import core
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.delete_dubbing_response_model import DeleteDubbingResponseModel
from ..types.do_dubbing_response import DoDubbingResponse
from ..types.dubbing_metadata_page_response_model import DubbingMetadataPageResponseModel
from ..types.dubbing_metadata_response import DubbingMetadataResponse
from .raw_client import AsyncRawDubbingClient, RawDubbingClient
from .types.dubbing_create_request_mode import DubbingCreateRequestMode
from .types.dubbing_list_request_creation_sources_item import DubbingListRequestCreationSourcesItem
from .types.dubbing_list_request_dubbing_models_item import DubbingListRequestDubbingModelsItem
from .types.dubbing_list_request_dubbing_status import DubbingListRequestDubbingStatus
from .types.dubbing_list_request_dubbing_statuses_item import DubbingListRequestDubbingStatusesItem
from .types.dubbing_list_request_filter_by_creator import DubbingListRequestFilterByCreator
from .types.dubbing_list_request_order_by import DubbingListRequestOrderBy
from .types.dubbing_list_request_order_direction import DubbingListRequestOrderDirection

if typing.TYPE_CHECKING:
    from .audio.client import AsyncAudioClient, AudioClient
    from .resource.client import AsyncResourceClient, ResourceClient
    from .transcript.client import AsyncTranscriptClient, TranscriptClient
    from .transcripts.client import AsyncTranscriptsClient, TranscriptsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DubbingClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDubbingClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._resource: typing.Optional[ResourceClient] = None
        self._audio: typing.Optional[AudioClient] = None
        self._transcript: typing.Optional[TranscriptClient] = None
        self._transcripts: typing.Optional[TranscriptsClient] = None

    @property
    def with_raw_response(self) -> RawDubbingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDubbingClient
        """
        return self._raw_client

    def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        dubbing_status: typing.Optional[DubbingListRequestDubbingStatus] = None,
        dubbing_statuses: typing.Optional[
            typing.Union[DubbingListRequestDubbingStatusesItem, typing.Sequence[DubbingListRequestDubbingStatusesItem]]
        ] = None,
        dubbing_models: typing.Optional[
            typing.Union[DubbingListRequestDubbingModelsItem, typing.Sequence[DubbingListRequestDubbingModelsItem]]
        ] = None,
        target_language_codes: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        creation_sources: typing.Optional[
            typing.Union[DubbingListRequestCreationSourcesItem, typing.Sequence[DubbingListRequestCreationSourcesItem]]
        ] = None,
        filter_by_creator: typing.Optional[DubbingListRequestFilterByCreator] = None,
        order_by: typing.Optional[DubbingListRequestOrderBy] = None,
        order_direction: typing.Optional[DubbingListRequestOrderDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DubbingMetadataPageResponseModel:
        """
        List the dubs you have access to.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        page_size : typing.Optional[int]
            How many dubs to return at maximum. Can not exceed 200, defaults to 100.

        dubbing_status : typing.Optional[DubbingListRequestDubbingStatus]
            What state the dub is currently in.

        dubbing_statuses : typing.Optional[typing.Union[DubbingListRequestDubbingStatusesItem, typing.Sequence[DubbingListRequestDubbingStatusesItem]]]
            Filter by dubbing status.

        dubbing_models : typing.Optional[typing.Union[DubbingListRequestDubbingModelsItem, typing.Sequence[DubbingListRequestDubbingModelsItem]]]
            Filter by dubbing model generation.

        target_language_codes : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter by target language code.

        creation_sources : typing.Optional[typing.Union[DubbingListRequestCreationSourcesItem, typing.Sequence[DubbingListRequestCreationSourcesItem]]]
            Filter by dubbing creation source.

        filter_by_creator : typing.Optional[DubbingListRequestFilterByCreator]
            Filters who created the resources being listed, whether it was the user running the request or someone else that shared the resource with them.

        order_by : typing.Optional[DubbingListRequestOrderBy]
            The field to use for ordering results from this query.

        order_direction : typing.Optional[DubbingListRequestOrderDirection]
            The order direction to use for results from this query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DubbingMetadataPageResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.dubbing.list(
            cursor="cursor",
            page_size=1,
            dubbing_status="dubbing",
            dubbing_statuses=["queued"],
            dubbing_models=["dubbing_v1"],
            target_language_codes=["target_language_codes"],
            creation_sources=["flow_node"],
            filter_by_creator="personal",
            order_by="created_at",
            order_direction="DESCENDING",
        )
        """
        _response = self._raw_client.list(
            cursor=cursor,
            page_size=page_size,
            dubbing_status=dubbing_status,
            dubbing_statuses=dubbing_statuses,
            dubbing_models=dubbing_models,
            target_language_codes=target_language_codes,
            creation_sources=creation_sources,
            filter_by_creator=filter_by_creator,
            order_by=order_by,
            order_direction=order_direction,
            request_options=request_options,
        )
        return _response.data

    def create(
        self,
        *,
        file: typing.Optional[core.File] = OMIT,
        csv_file: typing.Optional[core.File] = OMIT,
        foreground_audio_file: typing.Optional[core.File] = OMIT,
        background_audio_file: typing.Optional[core.File] = OMIT,
        name: typing.Optional[str] = OMIT,
        source_url: typing.Optional[str] = OMIT,
        source_lang: typing.Optional[str] = OMIT,
        target_lang: typing.Optional[str] = OMIT,
        target_accent: typing.Optional[str] = OMIT,
        num_speakers: typing.Optional[int] = OMIT,
        watermark: typing.Optional[bool] = OMIT,
        start_time: typing.Optional[int] = OMIT,
        end_time: typing.Optional[int] = OMIT,
        highest_resolution: typing.Optional[bool] = OMIT,
        drop_background_audio: typing.Optional[bool] = OMIT,
        use_profanity_filter: typing.Optional[bool] = OMIT,
        dubbing_studio: typing.Optional[bool] = OMIT,
        disable_voice_cloning: typing.Optional[bool] = OMIT,
        mode: typing.Optional[DubbingCreateRequestMode] = OMIT,
        csv_fps: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DoDubbingResponse:
        """
        Dubs a provided audio or video file into given language.

        Parameters
        ----------
        file : typing.Optional[core.File]
            See core.File for more documentation

        csv_file : typing.Optional[core.File]
            See core.File for more documentation

        foreground_audio_file : typing.Optional[core.File]
            See core.File for more documentation

        background_audio_file : typing.Optional[core.File]
            See core.File for more documentation

        name : typing.Optional[str]
            Name of the dubbing project.

        source_url : typing.Optional[str]
            URL of the source video/audio file.

        source_lang : typing.Optional[str]
            Source language. Expects a valid iso639-1 or iso639-3 language code.

        target_lang : typing.Optional[str]
            The Target language to dub the content into. Expects a valid iso639-1 or iso639-3 language code.

        target_accent : typing.Optional[str]
            [Experimental] An accent to apply when selecting voices from the library and to use to inform translation of the dialect to prefer.

        num_speakers : typing.Optional[int]
            Number of speakers to use for the dubbing. Set to 0 to automatically detect the number of speakers

        watermark : typing.Optional[bool]
            Whether to apply watermark to the output video.

        start_time : typing.Optional[int]
            Start time of the source video/audio file.

        end_time : typing.Optional[int]
            End time of the source video/audio file.

        highest_resolution : typing.Optional[bool]
            Whether to use the highest resolution available.

        drop_background_audio : typing.Optional[bool]
            An advanced setting. Whether to drop background audio from the final dub. This can improve dub quality where it's known that audio shouldn't have a background track such as for speeches or monologues.

        use_profanity_filter : typing.Optional[bool]
            [BETA] Whether transcripts should have profanities censored with the words '[censored]'

        dubbing_studio : typing.Optional[bool]
            Whether to prepare dub for edits in dubbing studio or edits as a dubbing resource.

        disable_voice_cloning : typing.Optional[bool]
            Instead of using a voice clone in dubbing, use a similar voice from the ElevenLabs Voice Library. Voices used from the library will contribute towards a workspace's custom voices limit, and if there aren't enough available slots the dub will fail. Using this feature requires the caller to have the 'add_voice_from_voice_library' permission on their workspace to access new voices.

        mode : typing.Optional[DubbingCreateRequestMode]
            The mode in which to run this Dubbing job. Defaults to automatic, use manual if specifically providing a CSV transcript to use. Note that manual mode is experimental and production use is strongly discouraged.

        csv_fps : typing.Optional[float]
            Frames per second to use when parsing a CSV file for dubbing. If not provided, FPS will be inferred from timecodes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DoDubbingResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.dubbing.create()
        """
        _response = self._raw_client.create(
            file=file,
            csv_file=csv_file,
            foreground_audio_file=foreground_audio_file,
            background_audio_file=background_audio_file,
            name=name,
            source_url=source_url,
            source_lang=source_lang,
            target_lang=target_lang,
            target_accent=target_accent,
            num_speakers=num_speakers,
            watermark=watermark,
            start_time=start_time,
            end_time=end_time,
            highest_resolution=highest_resolution,
            drop_background_audio=drop_background_audio,
            use_profanity_filter=use_profanity_filter,
            dubbing_studio=dubbing_studio,
            disable_voice_cloning=disable_voice_cloning,
            mode=mode,
            csv_fps=csv_fps,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, dubbing_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DubbingMetadataResponse:
        """
        Returns metadata about a dubbing project, including whether it's still in progress or not

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DubbingMetadataResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.dubbing.get(
            dubbing_id="dubbing_id",
        )
        """
        _response = self._raw_client.get(dubbing_id, request_options=request_options)
        return _response.data

    def delete(
        self, dubbing_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteDubbingResponseModel:
        """
        Deletes a dubbing project.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteDubbingResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.dubbing.delete(
            dubbing_id="dubbing_id",
        )
        """
        _response = self._raw_client.delete(dubbing_id, request_options=request_options)
        return _response.data

    @property
    def resource(self):
        if self._resource is None:
            from .resource.client import ResourceClient  # noqa: E402

            self._resource = ResourceClient(client_wrapper=self._client_wrapper)
        return self._resource

    @property
    def audio(self):
        if self._audio is None:
            from .audio.client import AudioClient  # noqa: E402

            self._audio = AudioClient(client_wrapper=self._client_wrapper)
        return self._audio

    @property
    def transcript(self):
        if self._transcript is None:
            from .transcript.client import TranscriptClient  # noqa: E402

            self._transcript = TranscriptClient(client_wrapper=self._client_wrapper)
        return self._transcript

    @property
    def transcripts(self):
        if self._transcripts is None:
            from .transcripts.client import TranscriptsClient  # noqa: E402

            self._transcripts = TranscriptsClient(client_wrapper=self._client_wrapper)
        return self._transcripts


class AsyncDubbingClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDubbingClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._resource: typing.Optional[AsyncResourceClient] = None
        self._audio: typing.Optional[AsyncAudioClient] = None
        self._transcript: typing.Optional[AsyncTranscriptClient] = None
        self._transcripts: typing.Optional[AsyncTranscriptsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawDubbingClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDubbingClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        cursor: typing.Optional[str] = None,
        page_size: typing.Optional[int] = None,
        dubbing_status: typing.Optional[DubbingListRequestDubbingStatus] = None,
        dubbing_statuses: typing.Optional[
            typing.Union[DubbingListRequestDubbingStatusesItem, typing.Sequence[DubbingListRequestDubbingStatusesItem]]
        ] = None,
        dubbing_models: typing.Optional[
            typing.Union[DubbingListRequestDubbingModelsItem, typing.Sequence[DubbingListRequestDubbingModelsItem]]
        ] = None,
        target_language_codes: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        creation_sources: typing.Optional[
            typing.Union[DubbingListRequestCreationSourcesItem, typing.Sequence[DubbingListRequestCreationSourcesItem]]
        ] = None,
        filter_by_creator: typing.Optional[DubbingListRequestFilterByCreator] = None,
        order_by: typing.Optional[DubbingListRequestOrderBy] = None,
        order_direction: typing.Optional[DubbingListRequestOrderDirection] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DubbingMetadataPageResponseModel:
        """
        List the dubs you have access to.

        Parameters
        ----------
        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        page_size : typing.Optional[int]
            How many dubs to return at maximum. Can not exceed 200, defaults to 100.

        dubbing_status : typing.Optional[DubbingListRequestDubbingStatus]
            What state the dub is currently in.

        dubbing_statuses : typing.Optional[typing.Union[DubbingListRequestDubbingStatusesItem, typing.Sequence[DubbingListRequestDubbingStatusesItem]]]
            Filter by dubbing status.

        dubbing_models : typing.Optional[typing.Union[DubbingListRequestDubbingModelsItem, typing.Sequence[DubbingListRequestDubbingModelsItem]]]
            Filter by dubbing model generation.

        target_language_codes : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter by target language code.

        creation_sources : typing.Optional[typing.Union[DubbingListRequestCreationSourcesItem, typing.Sequence[DubbingListRequestCreationSourcesItem]]]
            Filter by dubbing creation source.

        filter_by_creator : typing.Optional[DubbingListRequestFilterByCreator]
            Filters who created the resources being listed, whether it was the user running the request or someone else that shared the resource with them.

        order_by : typing.Optional[DubbingListRequestOrderBy]
            The field to use for ordering results from this query.

        order_direction : typing.Optional[DubbingListRequestOrderDirection]
            The order direction to use for results from this query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DubbingMetadataPageResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.dubbing.list(
                cursor="cursor",
                page_size=1,
                dubbing_status="dubbing",
                dubbing_statuses=["queued"],
                dubbing_models=["dubbing_v1"],
                target_language_codes=["target_language_codes"],
                creation_sources=["flow_node"],
                filter_by_creator="personal",
                order_by="created_at",
                order_direction="DESCENDING",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            cursor=cursor,
            page_size=page_size,
            dubbing_status=dubbing_status,
            dubbing_statuses=dubbing_statuses,
            dubbing_models=dubbing_models,
            target_language_codes=target_language_codes,
            creation_sources=creation_sources,
            filter_by_creator=filter_by_creator,
            order_by=order_by,
            order_direction=order_direction,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self,
        *,
        file: typing.Optional[core.File] = OMIT,
        csv_file: typing.Optional[core.File] = OMIT,
        foreground_audio_file: typing.Optional[core.File] = OMIT,
        background_audio_file: typing.Optional[core.File] = OMIT,
        name: typing.Optional[str] = OMIT,
        source_url: typing.Optional[str] = OMIT,
        source_lang: typing.Optional[str] = OMIT,
        target_lang: typing.Optional[str] = OMIT,
        target_accent: typing.Optional[str] = OMIT,
        num_speakers: typing.Optional[int] = OMIT,
        watermark: typing.Optional[bool] = OMIT,
        start_time: typing.Optional[int] = OMIT,
        end_time: typing.Optional[int] = OMIT,
        highest_resolution: typing.Optional[bool] = OMIT,
        drop_background_audio: typing.Optional[bool] = OMIT,
        use_profanity_filter: typing.Optional[bool] = OMIT,
        dubbing_studio: typing.Optional[bool] = OMIT,
        disable_voice_cloning: typing.Optional[bool] = OMIT,
        mode: typing.Optional[DubbingCreateRequestMode] = OMIT,
        csv_fps: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> DoDubbingResponse:
        """
        Dubs a provided audio or video file into given language.

        Parameters
        ----------
        file : typing.Optional[core.File]
            See core.File for more documentation

        csv_file : typing.Optional[core.File]
            See core.File for more documentation

        foreground_audio_file : typing.Optional[core.File]
            See core.File for more documentation

        background_audio_file : typing.Optional[core.File]
            See core.File for more documentation

        name : typing.Optional[str]
            Name of the dubbing project.

        source_url : typing.Optional[str]
            URL of the source video/audio file.

        source_lang : typing.Optional[str]
            Source language. Expects a valid iso639-1 or iso639-3 language code.

        target_lang : typing.Optional[str]
            The Target language to dub the content into. Expects a valid iso639-1 or iso639-3 language code.

        target_accent : typing.Optional[str]
            [Experimental] An accent to apply when selecting voices from the library and to use to inform translation of the dialect to prefer.

        num_speakers : typing.Optional[int]
            Number of speakers to use for the dubbing. Set to 0 to automatically detect the number of speakers

        watermark : typing.Optional[bool]
            Whether to apply watermark to the output video.

        start_time : typing.Optional[int]
            Start time of the source video/audio file.

        end_time : typing.Optional[int]
            End time of the source video/audio file.

        highest_resolution : typing.Optional[bool]
            Whether to use the highest resolution available.

        drop_background_audio : typing.Optional[bool]
            An advanced setting. Whether to drop background audio from the final dub. This can improve dub quality where it's known that audio shouldn't have a background track such as for speeches or monologues.

        use_profanity_filter : typing.Optional[bool]
            [BETA] Whether transcripts should have profanities censored with the words '[censored]'

        dubbing_studio : typing.Optional[bool]
            Whether to prepare dub for edits in dubbing studio or edits as a dubbing resource.

        disable_voice_cloning : typing.Optional[bool]
            Instead of using a voice clone in dubbing, use a similar voice from the ElevenLabs Voice Library. Voices used from the library will contribute towards a workspace's custom voices limit, and if there aren't enough available slots the dub will fail. Using this feature requires the caller to have the 'add_voice_from_voice_library' permission on their workspace to access new voices.

        mode : typing.Optional[DubbingCreateRequestMode]
            The mode in which to run this Dubbing job. Defaults to automatic, use manual if specifically providing a CSV transcript to use. Note that manual mode is experimental and production use is strongly discouraged.

        csv_fps : typing.Optional[float]
            Frames per second to use when parsing a CSV file for dubbing. If not provided, FPS will be inferred from timecodes.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DoDubbingResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.dubbing.create()


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            file=file,
            csv_file=csv_file,
            foreground_audio_file=foreground_audio_file,
            background_audio_file=background_audio_file,
            name=name,
            source_url=source_url,
            source_lang=source_lang,
            target_lang=target_lang,
            target_accent=target_accent,
            num_speakers=num_speakers,
            watermark=watermark,
            start_time=start_time,
            end_time=end_time,
            highest_resolution=highest_resolution,
            drop_background_audio=drop_background_audio,
            use_profanity_filter=use_profanity_filter,
            dubbing_studio=dubbing_studio,
            disable_voice_cloning=disable_voice_cloning,
            mode=mode,
            csv_fps=csv_fps,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, dubbing_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DubbingMetadataResponse:
        """
        Returns metadata about a dubbing project, including whether it's still in progress or not

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DubbingMetadataResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.dubbing.get(
                dubbing_id="dubbing_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(dubbing_id, request_options=request_options)
        return _response.data

    async def delete(
        self, dubbing_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteDubbingResponseModel:
        """
        Deletes a dubbing project.

        Parameters
        ----------
        dubbing_id : str
            ID of the dubbing project.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteDubbingResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.dubbing.delete(
                dubbing_id="dubbing_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(dubbing_id, request_options=request_options)
        return _response.data

    @property
    def resource(self):
        if self._resource is None:
            from .resource.client import AsyncResourceClient  # noqa: E402

            self._resource = AsyncResourceClient(client_wrapper=self._client_wrapper)
        return self._resource

    @property
    def audio(self):
        if self._audio is None:
            from .audio.client import AsyncAudioClient  # noqa: E402

            self._audio = AsyncAudioClient(client_wrapper=self._client_wrapper)
        return self._audio

    @property
    def transcript(self):
        if self._transcript is None:
            from .transcript.client import AsyncTranscriptClient  # noqa: E402

            self._transcript = AsyncTranscriptClient(client_wrapper=self._client_wrapper)
        return self._transcript

    @property
    def transcripts(self):
        if self._transcripts is None:
            from .transcripts.client import AsyncTranscriptsClient  # noqa: E402

            self._transcripts = AsyncTranscriptsClient(client_wrapper=self._client_wrapper)
        return self._transcripts



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .audio_format_enum import AudioFormatEnum
from .session_started_payload_config_commit_strategy import SessionStartedPayloadConfigCommitStrategy


class SessionStartedPayloadConfig(UncheckedBaseModel):
    """
    Configuration for the transcription session.
    """

    sample_rate: typing.Optional[int] = pydantic.Field(default=None)
    """
    Sample rate of the audio in Hz.
    """

    audio_format: typing.Optional[AudioFormatEnum] = None
    language_code: typing.Optional[str] = pydantic.Field(default=None)
    """
    Language code in ISO 639-1 or ISO 639-3 format.
    """

    commit_strategy: typing.Optional[SessionStartedPayloadConfigCommitStrategy] = pydantic.Field(default=None)
    """
    Strategy for committing transcriptions.
    """

    vad_silence_threshold_secs: typing.Optional[float] = pydantic.Field(default=None)
    """
    Silence threshold in seconds.
    """

    vad_threshold: typing.Optional[float] = pydantic.Field(default=None)
    """
    Threshold for voice activity detection.
    """

    min_speech_duration_ms: typing.Optional[int] = pydantic.Field(default=None)
    """
    Minimum speech duration in milliseconds.
    """

    min_silence_duration_ms: typing.Optional[int] = pydantic.Field(default=None)
    """
    Minimum silence duration in milliseconds.
    """

    model_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of the model to use for transcription.
    """

    enable_logging: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enable_logging is set to false zero retention mode will be used for the request. This will mean history features are unavailable for this request. Zero retention mode may only be used by enterprise customers.
    """

    include_timestamps: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the session will include word-level timestamps in the committed transcript.
    """

    include_language_detection: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the session will include language detection in the committed transcript.
    """

    keyterms: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of keyterms the model is biased towards.
    """

    no_verbatim: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether filler words and disfluencies are removed from the transcript.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class LlmDeprecationConfigModel(UncheckedBaseModel):
    warning_start_days: int = pydantic.Field()
    """
    Number of days before the provider deprecation date when warnings start being shown.
    """

    fallback_start_days: int = pydantic.Field()
    """
    Number of days before the provider deprecation date when traffic starts being routed to the replacement model.
    """

    fallback_complete_days: int = pydantic.Field()
    """
    Number of days before the provider deprecation date when all traffic is routed to the replacement model.
    """

    fallback_start_percentage: int = pydantic.Field()
    """
    Percentage of traffic routed to the replacement model when fallback begins.
    """

    fallback_complete_percentage: int = pydantic.Field()
    """
    Percentage of traffic routed to the replacement model when fallback is complete.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .spelling_patience import SpellingPatience
from .turn_eagerness import TurnEagerness
from .turn_model import TurnModel


class BaseTurnConfig(UncheckedBaseModel):
    turn_timeout: typing.Optional[float] = pydantic.Field(default=None)
    """
    Maximum wait time for the user's reply before re-engaging the user
    """

    initial_wait_time: typing.Optional[float] = pydantic.Field(default=None)
    """
    How long the agent will wait for the user to start the conversation if the first message is empty. If not set, uses the regular turn_timeout.
    """

    silence_end_call_timeout: typing.Optional[float] = pydantic.Field(default=None)
    """
    Maximum wait time since the user last spoke before terminating the call
    """

    turn_eagerness: typing.Optional[TurnEagerness] = pydantic.Field(default=None)
    """
    Controls how eager the agent is to respond. Low = less eager (waits longer), Standard = default eagerness, High = more eager (responds sooner)
    """

    spelling_patience: typing.Optional[SpellingPatience] = pydantic.Field(default=None)
    """
    Controls if the agent should be more patient when user is spelling numbers and named entities. Auto = model based, Off = never wait extra
    """

    speculative_turn: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, starts generating LLM responses during silence before full turn confidence is reached, reducing perceived latency. May increase LLM costs.
    """

    retranscribe_on_turn_timeout: typing.Optional[bool] = pydantic.Field(default=None)
    """
    When enabled, if VAD detects no speech, attempts to re-transcribe accumulated audio at turn timeout. Disables silence discount billing for affected turns.
    """

    turn_model: typing.Optional[TurnModel] = None
    interruption_ignore_terms: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of terms that should not trigger an interruption when spoken by the user (e.g. 'gotcha', 'understood'). Uses case-insensitive exact matching.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ProjectsCreateRequestSourceType = typing.Union[
    typing.Literal["blank", "book", "article", "genfm", "video", "screenplay"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .voice import Voice


class GetVoicesV2Response(UncheckedBaseModel):
    voices: typing.List[Voice] = pydantic.Field()
    """
    The list of voices matching the query.
    """

    has_more: bool = pydantic.Field()
    """
    Indicates whether there are more voices available in subsequent pages. Use this flag (and next_page_token) for reliable pagination instead of relying on total_count.
    """

    total_count: int = pydantic.Field()
    """
    The total count of voices matching the query. This value is a live snapshot that reflects the current state of the database and may change between requests as users create, modify, or delete voices. For reliable pagination, use the has_more flag instead of relying on this value. Only request this field when you actually need the total count (e.g., for display purposes), as calculating it incurs a performance cost.
    """

    next_page_token: typing.Optional[str] = pydantic.Field(default=None)
    """
    Token to retrieve the next page of results. Pass this value to the next request to continue pagination. Null if there are no more results.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

WebhookToolApiSchemaConfigOutputContentType = typing.Union[
    typing.Literal["application/json", "application/x-www-form-urlencoded"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .conversational_config_api_model_workflow_override_output import ConversationalConfigApiModelWorkflowOverrideOutput
from .entry_behavior import EntryBehavior
from .knowledge_base_locator import KnowledgeBaseLocator
from .position_output import PositionOutput


class WorkflowOverrideAgentNodeModelOutput(UncheckedBaseModel):
    conversation_config: ConversationalConfigApiModelWorkflowOverrideOutput = pydantic.Field()
    """
    Configuration overrides applied while the subagent is conducting the conversation.
    """

    additional_prompt: str = pydantic.Field()
    """
    Specific goal for this subagent. It will be added to the system prompt and can be used to further refine the agent's behavior in this specific context.
    """

    additional_knowledge_base: typing.List[KnowledgeBaseLocator] = pydantic.Field()
    """
    Additional knowledge base documents that the subagent has access to. These will be used in addition to the main agent's documents.
    """

    additional_tool_ids: typing.List[str] = pydantic.Field()
    """
    IDs of additional tools that the subagent has access to. These will be used in addition to the main agent's tools.
    """

    position: PositionOutput = pydantic.Field()
    """
    Position of the node in the workflow.
    """

    edge_order: typing.List[str] = pydantic.Field()
    """
    The ids of outgoing edges in the order they should be evaluated.
    """

    label: str = pydantic.Field()
    """
    Human-readable label for the node used throughout the UI.
    """

    entry_behavior: EntryBehavior = pydantic.Field()
    """
    Dictates whether this node should immediately generate a response upon entry or wait for the user input. When set to "auto", the behavior will be decided based on the type of the preceding node: "wait_for_user" after the "say" and "start" nodes and "generate_immediately" otherwise.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(WorkflowOverrideAgentNodeModelOutput)



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from .raw_client import AsyncRawAudioClient, RawAudioClient


class AudioClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAudioClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAudioClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAudioClient
        """
        return self._raw_client

    def get(
        self, voice_id: str, sample_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Iterator[bytes]:
        """
        Returns the audio corresponding to a sample attached to a voice.

        Parameters
        ----------
        voice_id : str
            ID of the voice to be used. You can use the [Get voices](/docs/api-reference/voices/search) endpoint list all the available voices.

        sample_id : str
            ID of the sample to be used. You can use the [Get voices](/docs/api-reference/voices/get) endpoint list all the available samples for a voice.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.Iterator[bytes]
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.voices.samples.audio.get(
            voice_id="voice_id",
            sample_id="sample_id",
        )
        """
        with self._raw_client.get(voice_id, sample_id, request_options=request_options) as r:
            yield from r.data


class AsyncAudioClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAudioClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAudioClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAudioClient
        """
        return self._raw_client

    async def get(
        self, voice_id: str, sample_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[bytes]:
        """
        Returns the audio corresponding to a sample attached to a voice.

        Parameters
        ----------
        voice_id : str
            ID of the voice to be used. You can use the [Get voices](/docs/api-reference/voices/search) endpoint list all the available voices.

        sample_id : str
            ID of the sample to be used. You can use the [Get voices](/docs/api-reference/voices/get) endpoint list all the available samples for a voice.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration. You can pass in configuration such as `chunk_size`, and more to customize the request and response.

        Returns
        -------
        typing.AsyncIterator[bytes]
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.voices.samples.audio.get(
                voice_id="voice_id",
                sample_id="sample_id",
            )


        asyncio.run(main())
        """
        async with self._raw_client.get(voice_id, sample_id, request_options=request_options) as r:
            async for _chunk in r.data:
                yield _chunk



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .asset_transcription import AssetTranscription
from .audio_analysis import AudioAnalysis
from .pending_blocks_metadata_model import PendingBlocksMetadataModel
from .pending_clip_task import PendingClipTask
from .pending_external_audios_metadata_model import PendingExternalAudiosMetadataModel
from .project_external_audio_response_model_source_context import ProjectExternalAudioResponseModelSourceContext


class ProjectExternalAudioResponseModel(UncheckedBaseModel):
    external_audio_id: str
    filename: str
    signed_url: typing.Optional[str] = None
    offset_ms: int
    duration_ms: int
    start_time_ms: int
    end_time_ms: typing.Optional[int] = None
    order: str
    track_id: str
    created_at_ms: int
    updated_at_ms: int
    volume_gain_db: typing.Optional[float] = None
    muted: typing.Optional[bool] = None
    fade_in_ms: typing.Optional[int] = None
    fade_out_ms: typing.Optional[int] = None
    source_external_audio_id: typing.Optional[str] = None
    source_asset_id: typing.Optional[str] = None
    source_platform_asset_id: typing.Optional[str] = None
    pending_blocks_metadata: typing.Optional[PendingBlocksMetadataModel] = None
    pending_external_audios_metadata: typing.Optional[PendingExternalAudiosMetadataModel] = None
    speech_imported: typing.Optional[bool] = None
    pending_task: typing.Optional[PendingClipTask] = None
    error: typing.Optional[str] = None
    current_snapshot_id: typing.Optional[str] = None
    source_context: typing.Optional[ProjectExternalAudioResponseModelSourceContext] = None
    analysis: typing.Optional[AudioAnalysis] = None
    transcription: typing.Optional[AssetTranscription] = None
    import_speech_progress: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.add_voice_response_model import AddVoiceResponseModel
from ...types.start_pvc_voice_training_response_model import StartPvcVoiceTrainingResponseModel

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPvcClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        name: str,
        language: str,
        description: typing.Optional[str] = OMIT,
        labels: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddVoiceResponseModel]:
        """
        Creates a new PVC voice with metadata but no samples

        Parameters
        ----------
        name : str
            The name that identifies this voice. This will be displayed in the dropdown of the website.

        language : str
            Language used in the samples.

        description : typing.Optional[str]
            Description to use for the created voice.

        labels : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Labels for the voice. Keys can be language, accent, gender, or age.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddVoiceResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/voices/pvc",
            method="POST",
            json={
                "name": name,
                "language": language,
                "description": description,
                "labels": labels,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddVoiceResponseModel,
                    construct_type(
                        type_=AddVoiceResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        voice_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        language: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        labels: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AddVoiceResponseModel]:
        """
        Edit PVC voice metadata

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        name : typing.Optional[str]
            The name that identifies this voice. This will be displayed in the dropdown of the website.

        language : typing.Optional[str]
            Language used in the samples.

        description : typing.Optional[str]
            Description to use for the created voice.

        labels : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Labels for the voice. Keys can be language, accent, gender, or age.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddVoiceResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/voices/pvc/{jsonable_encoder(voice_id)}",
            method="POST",
            json={
                "name": name,
                "language": language,
                "description": description,
                "labels": labels,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddVoiceResponseModel,
                    construct_type(
                        type_=AddVoiceResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def train(
        self,
        voice_id: str,
        *,
        model_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[StartPvcVoiceTrainingResponseModel]:
        """
        Start PVC training process for a voice.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        model_id : typing.Optional[str]
            The model ID to use for the conversion.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[StartPvcVoiceTrainingResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/voices/pvc/{jsonable_encoder(voice_id)}/train",
            method="POST",
            json={
                "model_id": model_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    StartPvcVoiceTrainingResponseModel,
                    construct_type(
                        type_=StartPvcVoiceTrainingResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPvcClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        name: str,
        language: str,
        description: typing.Optional[str] = OMIT,
        labels: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddVoiceResponseModel]:
        """
        Creates a new PVC voice with metadata but no samples

        Parameters
        ----------
        name : str
            The name that identifies this voice. This will be displayed in the dropdown of the website.

        language : str
            Language used in the samples.

        description : typing.Optional[str]
            Description to use for the created voice.

        labels : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Labels for the voice. Keys can be language, accent, gender, or age.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddVoiceResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/voices/pvc",
            method="POST",
            json={
                "name": name,
                "language": language,
                "description": description,
                "labels": labels,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddVoiceResponseModel,
                    construct_type(
                        type_=AddVoiceResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        voice_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        language: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        labels: typing.Optional[typing.Dict[str, typing.Optional[str]]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AddVoiceResponseModel]:
        """
        Edit PVC voice metadata

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        name : typing.Optional[str]
            The name that identifies this voice. This will be displayed in the dropdown of the website.

        language : typing.Optional[str]
            Language used in the samples.

        description : typing.Optional[str]
            Description to use for the created voice.

        labels : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Labels for the voice. Keys can be language, accent, gender, or age.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddVoiceResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/voices/pvc/{jsonable_encoder(voice_id)}",
            method="POST",
            json={
                "name": name,
                "language": language,
                "description": description,
                "labels": labels,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddVoiceResponseModel,
                    construct_type(
                        type_=AddVoiceResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def train(
        self,
        voice_id: str,
        *,
        model_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[StartPvcVoiceTrainingResponseModel]:
        """
        Start PVC training process for a voice.

        Parameters
        ----------
        voice_id : str
            Voice ID to be used, you can use https://api.elevenlabs.io/v1/voices to list all the available voices.

        model_id : typing.Optional[str]
            The model ID to use for the conversion.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[StartPvcVoiceTrainingResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/voices/pvc/{jsonable_encoder(voice_id)}/train",
            method="POST",
            json={
                "model_id": model_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    StartPvcVoiceTrainingResponseModel,
                    construct_type(
                        type_=StartPvcVoiceTrainingResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.request_options import RequestOptions
from .....core.unchecked_base_model import construct_type
from .....errors.unprocessable_entity_error import UnprocessableEntityError
from .....types.embedding_model_enum import EmbeddingModelEnum
from .....types.knowledge_base_document_chunk_response_model import KnowledgeBaseDocumentChunkResponseModel


class RawChunkClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        documentation_id: str,
        chunk_id: str,
        *,
        embedding_model: typing.Optional[EmbeddingModelEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[KnowledgeBaseDocumentChunkResponseModel]:
        """
        Get details about a specific documentation part used by RAG.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        chunk_id : str
            The id of a document RAG chunk from the knowledge base.

        embedding_model : typing.Optional[EmbeddingModelEnum]
            The embedding model used to retrieve the chunk.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[KnowledgeBaseDocumentChunkResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/knowledge-base/{jsonable_encoder(documentation_id)}/chunk/{jsonable_encoder(chunk_id)}",
            method="GET",
            params={
                "embedding_model": embedding_model,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseDocumentChunkResponseModel,
                    construct_type(
                        type_=KnowledgeBaseDocumentChunkResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawChunkClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        documentation_id: str,
        chunk_id: str,
        *,
        embedding_model: typing.Optional[EmbeddingModelEnum] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[KnowledgeBaseDocumentChunkResponseModel]:
        """
        Get details about a specific documentation part used by RAG.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        chunk_id : str
            The id of a document RAG chunk from the knowledge base.

        embedding_model : typing.Optional[EmbeddingModelEnum]
            The embedding model used to retrieve the chunk.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[KnowledgeBaseDocumentChunkResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/knowledge-base/{jsonable_encoder(documentation_id)}/chunk/{jsonable_encoder(chunk_id)}",
            method="GET",
            params={
                "embedding_model": embedding_model,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    KnowledgeBaseDocumentChunkResponseModel,
                    construct_type(
                        type_=KnowledgeBaseDocumentChunkResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ....types.multichannel_speech_to_text_response_model import MultichannelSpeechToTextResponseModel
from ....types.speech_to_text_chunk_response_model import SpeechToTextChunkResponseModel

TranscriptsGetResponse = typing.Union[SpeechToTextChunkResponseModel, MultichannelSpeechToTextResponseModel]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .llm_deprecation_config_model import LlmDeprecationConfigModel
from .llm_info_model_input import LlmInfoModelInput


class LlmListResponseModelInput(UncheckedBaseModel):
    llms: typing.List[LlmInfoModelInput] = pydantic.Field()
    """
    List of all available LLM models that can be used with agents.
    """

    default_deprecation_config: LlmDeprecationConfigModel = pydantic.Field()
    """
    The default deprecation timing configuration used for models without a custom override.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...types.environment_variable_auth_connection_value_request import EnvironmentVariableAuthConnectionValueRequest
from ...types.environment_variable_secret_value_request import EnvironmentVariableSecretValueRequest

UpdateEnvironmentVariableRequestValuesValue = typing.Union[
    str, EnvironmentVariableSecretValueRequest, EnvironmentVariableAuthConnectionValueRequest
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .document_usage_mode_enum import DocumentUsageModeEnum
from .external_file_sync_info import ExternalFileSyncInfo
from .knowledge_base_document_metadata_response_model import KnowledgeBaseDocumentMetadataResponseModel
from .knowledge_base_folder_path_segment_response_model import KnowledgeBaseFolderPathSegmentResponseModel
from .resource_access_info import ResourceAccessInfo


class GetKnowledgeBaseFileResponseModel(UncheckedBaseModel):
    id: str
    name: str
    metadata: KnowledgeBaseDocumentMetadataResponseModel
    supported_usages: typing.List[DocumentUsageModeEnum]
    access_info: ResourceAccessInfo
    folder_parent_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the parent folder, or null if the document is at the root level.
    """

    folder_path: typing.Optional[typing.List[KnowledgeBaseFolderPathSegmentResponseModel]] = pydantic.Field(
        default=None
    )
    """
    The folder path segments leading to this entity, from root to parent folder.
    """

    extracted_inner_html: str
    filename: str
    external_sync_info: typing.Optional[ExternalFileSyncInfo] = None
    is_frozen: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

PendingClipTaskType = typing.Union[
    typing.Literal["preprocessing", "speech_import", "dubbing", "video_to_music", "media_generation"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .mcp_server_config_output import McpServerConfigOutput
from .mcp_server_metadata_response_model import McpServerMetadataResponseModel
from .mcp_server_response_model_dependent_agents_item import McpServerResponseModelDependentAgentsItem
from .resource_access_info import ResourceAccessInfo


class McpServerResponseModel(UncheckedBaseModel):
    """
    Response model representing an MCP Server configuration.
    """

    id: str
    config: McpServerConfigOutput
    access_info: typing.Optional[ResourceAccessInfo] = pydantic.Field(default=None)
    """
    The access information of the MCP Server
    """

    dependent_agents: typing.Optional[typing.List[McpServerResponseModelDependentAgentsItem]] = pydantic.Field(
        default=None
    )
    """
    List of agents that depend on this MCP Server.
    """

    metadata: McpServerMetadataResponseModel = pydantic.Field()
    """
    The metadata of the MCP Server
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import os
import time
import asyncio

import pytest
from elevenlabs import ElevenLabs
from elevenlabs.conversational_ai.conversation import Conversation, ClientTools
from elevenlabs.conversational_ai.default_audio_interface import DefaultAudioInterface


@pytest.mark.skipif(os.getenv("CI") == "true", reason="Skip live conversation test in CI environment")
def test_live_conversation():
    """Test a live conversation with actual audio I/O"""

    api_key = os.getenv("ELEVENLABS_API_KEY")
    if not api_key:
        raise ValueError("ELEVENLABS_API_KEY environment variable missing.")

    agent_id = os.getenv("AGENT_ID")
    if not api_key or not agent_id:
        raise ValueError("AGENT_ID environment variable missing.")

    client = ElevenLabs(api_key=api_key)

    # Create conversation handlers
    def on_agent_response(text: str):
        print(f"Agent: {text}")

    def on_user_transcript(text: str):
        print(f"You: {text}")

    def on_latency(ms: int):
        print(f"Latency: {ms}ms")

    # Initialize client tools
    client_tools = ClientTools()

    def test(parameters):
        print("Sync tool called with parameters:", parameters)
        return "Tool called successfully"

    async def test_async(parameters):
        # Simulate some async work
        await asyncio.sleep(10)
        print("Async tool called with parameters:", parameters)
        return "Tool called successfully"

    client_tools.register("test", test)
    client_tools.register("test_async", test_async, is_async=True)

    # Initialize conversation
    conversation = Conversation(
        client=client,
        agent_id=agent_id,
        requires_auth=False,
        audio_interface=DefaultAudioInterface(),
        callback_agent_response=on_agent_response,
        callback_user_transcript=on_user_transcript,
        callback_latency_measurement=on_latency,
        client_tools=client_tools,
    )

    # Start the conversation
    conversation.start_session()

    # Let it run for 100 seconds
    time.sleep(100)

    # End the conversation
    conversation.end_session()
    conversation.wait_for_session_end()

    # Get the conversation ID for reference
    conversation_id = conversation._conversation_id
    print(f"Conversation ID: {conversation_id}")


if __name__ == "__main__":
    test_live_conversation()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .sample_config_db_model_parent_type import SampleConfigDbModelParentType


class SampleConfigDbModel(UncheckedBaseModel):
    is_sample: typing.Optional[bool] = None
    parent_id: typing.Optional[str] = None
    parent_type: typing.Optional[SampleConfigDbModelParentType] = None
    chapter_ids: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ResourceAccessInfoAccessSource = typing.Union[
    typing.Literal["creator", "explicit", "workspace_admin", "workspace_default"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import datetime as dt
import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.create_order_request import CreateOrderRequest
from ...types.create_order_response import CreateOrderResponse
from ...types.list_orders_response import ListOrdersResponse
from ...types.order_id import OrderId
from ...types.order_request_state import OrderRequestState
from ...types.order_response import OrderResponse
from ...types.submit_order_response import SubmitOrderResponse
from ...types.update_order_request import UpdateOrderRequest
from ...types.update_order_response import UpdateOrderResponse
from .raw_client import AsyncRawOrdersClient, RawOrdersClient

if typing.TYPE_CHECKING:
    from .deliverables.client import AsyncDeliverablesClient, DeliverablesClient
    from .items.client import AsyncItemsClient, ItemsClient
    from .languages.client import AsyncLanguagesClient, LanguagesClient
    from .media.client import AsyncMediaClient, MediaClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OrdersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawOrdersClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._media: typing.Optional[MediaClient] = None
        self._items: typing.Optional[ItemsClient] = None
        self._deliverables: typing.Optional[DeliverablesClient] = None
        self._languages: typing.Optional[LanguagesClient] = None

    @property
    def with_raw_response(self) -> RawOrdersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawOrdersClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        status: typing.Optional[typing.Union[OrderRequestState, typing.Sequence[OrderRequestState]]] = None,
        start_date: typing.Optional[dt.datetime] = None,
        end_date: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListOrdersResponse:
        """
        Lists Productions orders in the workspace. Supports filtering by status and date range, with pagination.

        Parameters
        ----------
        page_size : typing.Optional[int]
            Maximum number of orders to return per page.

        offset : typing.Optional[int]
            Number of orders to skip for pagination.

        status : typing.Optional[typing.Union[OrderRequestState, typing.Sequence[OrderRequestState]]]
            Filter orders by one or more statuses.

        start_date : typing.Optional[dt.datetime]
            Filter orders created on or after this date.

        end_date : typing.Optional[dt.datetime]
            Filter orders created on or before this date.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListOrdersResponse
            Successful Response

        Examples
        --------
        import datetime

        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.productions.orders.list(
            page_size=1,
            offset=1,
            status=["open"],
            start_date=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
            end_date=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
        )
        """
        _response = self._raw_client.list(
            page_size=page_size,
            offset=offset,
            status=status,
            start_date=start_date,
            end_date=end_date,
            request_options=request_options,
        )
        return _response.data

    def create(
        self,
        *,
        request: typing.Optional[CreateOrderRequest] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrderResponse:
        """
        Creates a new Productions order in the workspace. The order starts in the open state and can be configured with items before submission.

        Parameters
        ----------
        request : typing.Optional[CreateOrderRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrderResponse
            Successful Response

        Examples
        --------
        from elevenlabs import CreateOrderRequest, ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.productions.orders.create(
            request=CreateOrderRequest(
                sandbox=False,
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def get(self, order_id: OrderId, *, request_options: typing.Optional[RequestOptions] = None) -> OrderResponse:
        """
        Retrieves full details for a Productions order.

        Quote and pricing information may not be available immediately; if you wish to see the quote before submission, you may need to poll the order details until it is ready.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrderResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.productions.orders.get(
            order_id="order_id",
        )
        """
        _response = self._raw_client.get(order_id, request_options=request_options)
        return _response.data

    def update(
        self, order_id: OrderId, *, request: UpdateOrderRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateOrderResponse:
        """
        Updates an open order.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request : UpdateOrderRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrderResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs, UpdateOrderRequest

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.productions.orders.update(
            order_id="order_id",
            request=UpdateOrderRequest(
                name="Spanish Dubs",
            ),
        )
        """
        _response = self._raw_client.update(order_id, request=request, request_options=request_options)
        return _response.data

    def submit(
        self, order_id: OrderId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SubmitOrderResponse:
        """
        Submits an open order for processing. The order must have at least one item. Once submitted, items can no longer be modified.

        Upon submission, the workspace will be charged for the order. The quote is based on information extracted from the uploaded media, such as its duration. The quote may not be available immediately; if you wish to see the quote before submission, you may need to poll the order details until the quote is ready.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubmitOrderResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.productions.orders.submit(
            order_id="order_id",
        )
        """
        _response = self._raw_client.submit(order_id, request_options=request_options)
        return _response.data

    @property
    def media(self):
        if self._media is None:
            from .media.client import MediaClient  # noqa: E402

            self._media = MediaClient(client_wrapper=self._client_wrapper)
        return self._media

    @property
    def items(self):
        if self._items is None:
            from .items.client import ItemsClient  # noqa: E402

            self._items = ItemsClient(client_wrapper=self._client_wrapper)
        return self._items

    @property
    def deliverables(self):
        if self._deliverables is None:
            from .deliverables.client import DeliverablesClient  # noqa: E402

            self._deliverables = DeliverablesClient(client_wrapper=self._client_wrapper)
        return self._deliverables

    @property
    def languages(self):
        if self._languages is None:
            from .languages.client import LanguagesClient  # noqa: E402

            self._languages = LanguagesClient(client_wrapper=self._client_wrapper)
        return self._languages


class AsyncOrdersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawOrdersClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._media: typing.Optional[AsyncMediaClient] = None
        self._items: typing.Optional[AsyncItemsClient] = None
        self._deliverables: typing.Optional[AsyncDeliverablesClient] = None
        self._languages: typing.Optional[AsyncLanguagesClient] = None

    @property
    def with_raw_response(self) -> AsyncRawOrdersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawOrdersClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        offset: typing.Optional[int] = None,
        status: typing.Optional[typing.Union[OrderRequestState, typing.Sequence[OrderRequestState]]] = None,
        start_date: typing.Optional[dt.datetime] = None,
        end_date: typing.Optional[dt.datetime] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ListOrdersResponse:
        """
        Lists Productions orders in the workspace. Supports filtering by status and date range, with pagination.

        Parameters
        ----------
        page_size : typing.Optional[int]
            Maximum number of orders to return per page.

        offset : typing.Optional[int]
            Number of orders to skip for pagination.

        status : typing.Optional[typing.Union[OrderRequestState, typing.Sequence[OrderRequestState]]]
            Filter orders by one or more statuses.

        start_date : typing.Optional[dt.datetime]
            Filter orders created on or after this date.

        end_date : typing.Optional[dt.datetime]
            Filter orders created on or before this date.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListOrdersResponse
            Successful Response

        Examples
        --------
        import asyncio
        import datetime

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.productions.orders.list(
                page_size=1,
                offset=1,
                status=["open"],
                start_date=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                end_date=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            page_size=page_size,
            offset=offset,
            status=status,
            start_date=start_date,
            end_date=end_date,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self,
        *,
        request: typing.Optional[CreateOrderRequest] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateOrderResponse:
        """
        Creates a new Productions order in the workspace. The order starts in the open state and can be configured with items before submission.

        Parameters
        ----------
        request : typing.Optional[CreateOrderRequest]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateOrderResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs, CreateOrderRequest

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.productions.orders.create(
                request=CreateOrderRequest(
                    sandbox=False,
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def get(self, order_id: OrderId, *, request_options: typing.Optional[RequestOptions] = None) -> OrderResponse:
        """
        Retrieves full details for a Productions order.

        Quote and pricing information may not be available immediately; if you wish to see the quote before submission, you may need to poll the order details until it is ready.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrderResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.productions.orders.get(
                order_id="order_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(order_id, request_options=request_options)
        return _response.data

    async def update(
        self, order_id: OrderId, *, request: UpdateOrderRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> UpdateOrderResponse:
        """
        Updates an open order.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request : UpdateOrderRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateOrderResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs, UpdateOrderRequest

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.productions.orders.update(
                order_id="order_id",
                request=UpdateOrderRequest(
                    name="Spanish Dubs",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(order_id, request=request, request_options=request_options)
        return _response.data

    async def submit(
        self, order_id: OrderId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> SubmitOrderResponse:
        """
        Submits an open order for processing. The order must have at least one item. Once submitted, items can no longer be modified.

        Upon submission, the workspace will be charged for the order. The quote is based on information extracted from the uploaded media, such as its duration. The quote may not be available immediately; if you wish to see the quote before submission, you may need to poll the order details until the quote is ready.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SubmitOrderResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.productions.orders.submit(
                order_id="order_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.submit(order_id, request_options=request_options)
        return _response.data

    @property
    def media(self):
        if self._media is None:
            from .media.client import AsyncMediaClient  # noqa: E402

            self._media = AsyncMediaClient(client_wrapper=self._client_wrapper)
        return self._media

    @property
    def items(self):
        if self._items is None:
            from .items.client import AsyncItemsClient  # noqa: E402

            self._items = AsyncItemsClient(client_wrapper=self._client_wrapper)
        return self._items

    @property
    def deliverables(self):
        if self._deliverables is None:
            from .deliverables.client import AsyncDeliverablesClient  # noqa: E402

            self._deliverables = AsyncDeliverablesClient(client_wrapper=self._client_wrapper)
        return self._deliverables

    @property
    def languages(self):
        if self._languages is None:
            from .languages.client import AsyncLanguagesClient  # noqa: E402

            self._languages = AsyncLanguagesClient(client_wrapper=self._client_wrapper)
        return self._languages



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .tool import Tool


class ListMcpToolsResponseModel(UncheckedBaseModel):
    """
    Response model for testing tools available on an MCP server.
    """

    success: bool = pydantic.Field()
    """
    Indicates if the operation was successful.
    """

    tools: typing.List[Tool] = pydantic.Field()
    """
    A list of tools available on the MCP server.
    """

    error_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message if the operation was not successful.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        ConversationsGetRequestFormat,
        ConversationsListRequestExcludeStatusesItem,
        ConversationsListRequestSummaryMode,
    )
    from . import analysis, audio, feedback, files, messages, tags, topics
    from .messages import MessagesTextSearchRequestSummaryMode
_dynamic_imports: typing.Dict[str, str] = {
    "ConversationsGetRequestFormat": ".types",
    "ConversationsListRequestExcludeStatusesItem": ".types",
    "ConversationsListRequestSummaryMode": ".types",
    "MessagesTextSearchRequestSummaryMode": ".messages",
    "analysis": ".analysis",
    "audio": ".audio",
    "feedback": ".feedback",
    "files": ".files",
    "messages": ".messages",
    "tags": ".tags",
    "topics": ".topics",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "ConversationsGetRequestFormat",
    "ConversationsListRequestExcludeStatusesItem",
    "ConversationsListRequestSummaryMode",
    "MessagesTextSearchRequestSummaryMode",
    "analysis",
    "audio",
    "feedback",
    "files",
    "messages",
    "tags",
    "topics",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawTranscriptsClient, RawTranscriptsClient
from .types.transcripts_get_response import TranscriptsGetResponse


class TranscriptsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTranscriptsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTranscriptsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTranscriptsClient
        """
        return self._raw_client

    def get(
        self, transcription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TranscriptsGetResponse:
        """
        Retrieve a previously generated transcript by its ID.

        Parameters
        ----------
        transcription_id : str
            The unique ID of the transcript to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TranscriptsGetResponse
            The transcript data

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.speech_to_text.transcripts.get(
            transcription_id="transcription_id",
        )
        """
        _response = self._raw_client.get(transcription_id, request_options=request_options)
        return _response.data

    def delete(self, transcription_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> typing.Any:
        """
        Delete a previously generated transcript by its ID.

        Parameters
        ----------
        transcription_id : str
            The unique ID of the transcript to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Delete completed successfully.

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.speech_to_text.transcripts.delete(
            transcription_id="transcription_id",
        )
        """
        _response = self._raw_client.delete(transcription_id, request_options=request_options)
        return _response.data


class AsyncTranscriptsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTranscriptsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTranscriptsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTranscriptsClient
        """
        return self._raw_client

    async def get(
        self, transcription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TranscriptsGetResponse:
        """
        Retrieve a previously generated transcript by its ID.

        Parameters
        ----------
        transcription_id : str
            The unique ID of the transcript to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TranscriptsGetResponse
            The transcript data

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.speech_to_text.transcripts.get(
                transcription_id="transcription_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(transcription_id, request_options=request_options)
        return _response.data

    async def delete(
        self, transcription_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Any:
        """
        Delete a previously generated transcript by its ID.

        Parameters
        ----------
        transcription_id : str
            The unique ID of the transcript to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Delete completed successfully.

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.speech_to_text.transcripts.delete(
                transcription_id="transcription_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(transcription_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class CanvasPlacement(UncheckedBaseModel):
    """
    Defines asset positioning and transformation on canvas.
    """

    x_relative: typing.Optional[float] = None
    y_relative: typing.Optional[float] = None
    scale_x: typing.Optional[float] = None
    scale_y: typing.Optional[float] = None
    pivot_x: typing.Optional[float] = None
    pivot_y: typing.Optional[float] = None
    skew_x: typing.Optional[float] = None
    skew_y: typing.Optional[float] = None
    crop_top: typing.Optional[float] = None
    crop_right: typing.Optional[float] = None
    crop_bottom: typing.Optional[float] = None
    crop_left: typing.Optional[float] = None
    flip_x: typing.Optional[bool] = None
    flip_y: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawSamplesClient, RawSamplesClient

if typing.TYPE_CHECKING:
    from .audio.client import AsyncAudioClient, AudioClient


class SamplesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSamplesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._audio: typing.Optional[AudioClient] = None

    @property
    def with_raw_response(self) -> RawSamplesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSamplesClient
        """
        return self._raw_client

    @property
    def audio(self):
        if self._audio is None:
            from .audio.client import AudioClient  # noqa: E402

            self._audio = AudioClient(client_wrapper=self._client_wrapper)
        return self._audio


class AsyncSamplesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSamplesClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._audio: typing.Optional[AsyncAudioClient] = None

    @property
    def with_raw_response(self) -> AsyncRawSamplesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSamplesClient
        """
        return self._raw_client

    @property
    def audio(self):
        if self._audio is None:
            from .audio.client import AsyncAudioClient  # noqa: E402

            self._audio = AsyncAudioClient(client_wrapper=self._client_wrapper)
        return self._audio



# This file was auto-generated by Fern from our API Definition.

import typing

VoicesGetSharedRequestSort = typing.Union[
    typing.Literal["created_date", "usage_character_count_1y", "trending", "cloned_by_count"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

ToolCallSoundType = typing.Union[
    typing.Literal["typing", "elevator1", "elevator2", "elevator3", "elevator4"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .agent_workflow_request_model_nodes_value import AgentWorkflowRequestModelNodesValue
from .workflow_edge_model_input import WorkflowEdgeModelInput


class AgentWorkflowRequestModel(UncheckedBaseModel):
    edges: typing.Optional[typing.Dict[str, WorkflowEdgeModelInput]] = None
    nodes: typing.Optional[typing.Dict[str, AgentWorkflowRequestModelNodesValue]] = None
    prevent_subagent_loops: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether to prevent loops in the workflow execution.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(AgentWorkflowRequestModel)



# This file was auto-generated by Fern from our API Definition.

import typing

from .auth_connection_locator import AuthConnectionLocator
from .environment_auth_connection_locator import EnvironmentAuthConnectionLocator

CustomLlmAuthConnection = typing.Union[AuthConnectionLocator, EnvironmentAuthConnectionLocator]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .webhook_event_type import WebhookEventType
from .webhook_transcript_format import WebhookTranscriptFormat


class ConvAiWebhooks(UncheckedBaseModel):
    post_call_webhook_id: typing.Optional[str] = None
    events: typing.Optional[typing.List[WebhookEventType]] = pydantic.Field(default=None)
    """
    List of event types to send via webhook. Options: transcript, audio, call_initiation_failure.
    """

    transcript_format: typing.Optional[WebhookTranscriptFormat] = pydantic.Field(default=None)
    """
    Format for transcript webhooks.
    """

    send_audio: typing.Optional[bool] = pydantic.Field(default=None)
    """
    DEPRECATED: Use 'events' field instead. Whether to send audio data with post-call webhooks for ConvAI conversations
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawApiKeysClient, RawApiKeysClient


class ApiKeysClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawApiKeysClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawApiKeysClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawApiKeysClient
        """
        return self._raw_client

    def disable(self, *, api_key_name: str, request_options: typing.Optional[RequestOptions] = None) -> typing.Any:
        """
        Disable the API key used to authenticate this request. Requires the query parameter `api_key_name=self` as an explicit confirmation. This endpoint requires additional permissions and is not enabled by default. Reach out to your ElevenLabs contact to request access.

        Parameters
        ----------
        api_key_name : str
            Must be set to `self` to disable the API key used to authenticate this request. Required as an explicit confirmation to avoid accidentally disabling the wrong key.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.workspaces.api_keys.disable(
            api_key_name="self",
        )
        """
        _response = self._raw_client.disable(api_key_name=api_key_name, request_options=request_options)
        return _response.data


class AsyncApiKeysClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawApiKeysClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawApiKeysClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawApiKeysClient
        """
        return self._raw_client

    async def disable(
        self, *, api_key_name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Any:
        """
        Disable the API key used to authenticate this request. Requires the query parameter `api_key_name=self` as an explicit confirmation. This endpoint requires additional permissions and is not enabled by default. Reach out to your ElevenLabs contact to request access.

        Parameters
        ----------
        api_key_name : str
            Must be set to `self` to disable the API key used to authenticate this request. Required as an explicit confirmation to avoid accidentally disabling the wrong key.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.workspaces.api_keys.disable(
                api_key_name="self",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.disable(api_key_name=api_key_name, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .speech_engine_config_request_headers_value import SpeechEngineConfigRequestHeadersValue


class SpeechEngineConfig(UncheckedBaseModel):
    ws_url: str = pydantic.Field()
    """
    The WebSocket URL for the transcript server
    """

    request_headers: typing.Optional[typing.Dict[str, SpeechEngineConfigRequestHeadersValue]] = pydantic.Field(
        default=None
    )
    """
    Headers to include in the WebSocket connection request
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import json
import time
import hmac
import hashlib
from unittest import mock

from elevenlabs.client import ElevenLabs
from elevenlabs.errors import BadRequestError

import pytest


def test_construct_event_valid_signature():
    """Test webhook event construction with valid signature."""
    # Setup
    client = ElevenLabs()
    webhook_secret = "test_secret"
    payload = {"event_type": "speech.completed", "id": "123456"}

    # Create a valid signature
    body = json.dumps(payload)
    timestamp = str(int(time.time()))
    message = f"{timestamp}.{body}"
    signature = "v0=" + hmac.new(
        webhook_secret.encode('utf-8'),
        message.encode('utf-8'),
        hashlib.sha256
    ).hexdigest()
    sig_header = f"t={timestamp},{signature}"

    # Verify event construction
    event = client.webhooks.construct_event(body, sig_header, webhook_secret)
    assert event == payload, "Event should match the original payload"


def test_construct_event_missing_signature():
    """Test webhook event construction with missing signature header."""
    client = ElevenLabs()
    webhook_secret = "test_secret"
    payload = {"event_type": "speech.completed", "id": "123456"}

    with pytest.raises(BadRequestError) as excinfo:
        client.webhooks.construct_event(payload, "", webhook_secret)

    assert "Missing signature header" in str(excinfo.value)


def test_construct_event_invalid_signature_format():
    """Test webhook event construction with invalid signature format."""
    client = ElevenLabs()
    webhook_secret = "test_secret"
    payload = {"event_type": "speech.completed", "id": "123456"}
    body = json.dumps(payload)
    sig_header = "invalid_format"

    with pytest.raises(BadRequestError) as excinfo:
        client.webhooks.construct_event(body, sig_header, webhook_secret)

    assert "No signature hash found with expected scheme v0" in str(excinfo.value)


def test_construct_event_expired_timestamp():
    """Test webhook event construction with expired timestamp."""
    client = ElevenLabs()
    webhook_secret = "test_secret"
    payload = {"event_type": "speech.completed", "id": "123456"}

    # Create an expired timestamp (31 minutes old)
    expired_time = int(time.time()) - 31 * 60
    timestamp = str(expired_time)

    body = json.dumps(payload)
    message = f"{timestamp}.{body}"
    signature = "v0=" + hmac.new(
        webhook_secret.encode('utf-8'),
        message.encode('utf-8'),
        hashlib.sha256
    ).hexdigest()
    sig_header = f"t={timestamp},{signature}"

    with pytest.raises(BadRequestError) as excinfo:
        client.webhooks.construct_event(body, sig_header, webhook_secret)

    assert "Timestamp outside the tolerance zone" in str(excinfo.value)


def test_construct_event_invalid_signature():
    """Test webhook event construction with invalid signature."""
    client = ElevenLabs()
    webhook_secret = "test_secret"
    payload = {"event_type": "speech.completed", "id": "123456"}
    body = json.dumps(payload)

    timestamp = str(int(time.time()))
    sig_header = f"t={timestamp},v0=invalid_signature"

    with pytest.raises(BadRequestError) as excinfo:
        client.webhooks.construct_event(body, sig_header, webhook_secret)

    assert "Signature hash does not match" in str(excinfo.value)


def test_construct_event_missing_secret():
    """Test webhook event construction with missing secret."""
    client = ElevenLabs()
    payload = {"event_type": "speech.completed", "id": "123456"}
    body = json.dumps(payload)

    timestamp = str(int(time.time()))
    sig_header = f"t={timestamp},v0=some_signature"

    with pytest.raises(BadRequestError) as excinfo:
        client.webhooks.construct_event(body, sig_header, "")

    assert "Webhook secret not configured" in str(excinfo.value)


@mock.patch('time.time')
def test_construct_event_mocked_time(mock_time):
    """Test webhook event construction with mocked time."""
    mock_time.return_value = 1600000000

    client = ElevenLabs()
    webhook_secret = "test_secret"
    payload = {"event_type": "speech.completed", "id": "123456"}

    # Create a valid signature with fixed timestamp
    body = json.dumps(payload)
    timestamp = "1600000000"
    message = f"{timestamp}.{body}"
    signature = "v0=" + hmac.new(
        webhook_secret.encode('utf-8'),
        message.encode('utf-8'),
        hashlib.sha256
    ).hexdigest()
    sig_header = f"t={timestamp},{signature}"

    # Verify event construction
    event = client.webhooks.construct_event(body, sig_header, webhook_secret)
    assert event == payload, "Event should match the original payload"



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .caption_style_character_animation_model import CaptionStyleCharacterAnimationModel
from .caption_style_horizontal_placement_model import CaptionStyleHorizontalPlacementModel
from .caption_style_model_text_align import CaptionStyleModelTextAlign
from .caption_style_model_text_blend_mode import CaptionStyleModelTextBlendMode
from .caption_style_model_text_style import CaptionStyleModelTextStyle
from .caption_style_model_text_transform import CaptionStyleModelTextTransform
from .caption_style_model_text_weight import CaptionStyleModelTextWeight
from .caption_style_section_animation_model import CaptionStyleSectionAnimationModel
from .caption_style_template_model import CaptionStyleTemplateModel
from .caption_style_vertical_placement_model import CaptionStyleVerticalPlacementModel
from .caption_style_word_animation_model import CaptionStyleWordAnimationModel
from .studio_text_style_outline_model import StudioTextStyleOutlineModel
from .studio_text_style_shadow_model import StudioTextStyleShadowModel


class CaptionStyleModel(UncheckedBaseModel):
    template: typing.Optional[CaptionStyleTemplateModel] = None
    text_font: typing.Optional[str] = None
    text_scale: typing.Optional[float] = None
    text_color: typing.Optional[str] = None
    text_align: typing.Optional[CaptionStyleModelTextAlign] = None
    text_style: typing.Optional[CaptionStyleModelTextStyle] = None
    text_weight: typing.Optional[CaptionStyleModelTextWeight] = None
    text_transform: typing.Optional[CaptionStyleModelTextTransform] = None
    text_blend_mode: typing.Optional[CaptionStyleModelTextBlendMode] = None
    text_shadow: typing.Optional[StudioTextStyleShadowModel] = None
    text_outline: typing.Optional[StudioTextStyleOutlineModel] = None
    background_enabled: typing.Optional[bool] = None
    background_color: typing.Optional[str] = None
    background_opacity: typing.Optional[float] = None
    word_highlights_enabled: typing.Optional[bool] = None
    word_highlights_color: typing.Optional[str] = None
    word_highlights_background_color: typing.Optional[str] = None
    word_highlights_opacity: typing.Optional[float] = None
    section_animation: typing.Optional[CaptionStyleSectionAnimationModel] = None
    word_animation: typing.Optional[CaptionStyleWordAnimationModel] = None
    character_animation: typing.Optional[CaptionStyleCharacterAnimationModel] = None
    width_pct: typing.Optional[float] = None
    horizontal_placement: typing.Optional[CaptionStyleHorizontalPlacementModel] = None
    vertical_placement: typing.Optional[CaptionStyleVerticalPlacementModel] = None
    auto_break_enabled: typing.Optional[bool] = None
    max_lines_per_section: typing.Optional[int] = None
    max_words_per_line: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class KeepContextAlive(UncheckedBaseModel):
    """
    Payload to keep a specific context alive by resetting its inactivity timeout. Empty text is ignored but resets the clock.
    """

    text: typing.Literal[""] = pydantic.Field(default="")
    """
    An empty string. This text is ignored by the server but its presence resets the inactivity timeout for the specified context.
    """

    context_id: str = pydantic.Field()
    """
    The identifier of the context to keep alive.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .conv_ai_env_var_locator import ConvAiEnvVarLocator
from .conv_ai_secret_locator import ConvAiSecretLocator

CustomLlmApiKey = typing.Union[ConvAiSecretLocator, ConvAiEnvVarLocator]



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.batch_call_detailed_response import BatchCallDetailedResponse
from ...types.batch_call_response import BatchCallResponse
from ...types.batch_call_whats_app_params import BatchCallWhatsAppParams
from ...types.outbound_call_recipient import OutboundCallRecipient
from ...types.telephony_call_config import TelephonyCallConfig
from ...types.workspace_batch_calls_response import WorkspaceBatchCallsResponse
from .raw_client import AsyncRawBatchCallsClient, RawBatchCallsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class BatchCallsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawBatchCallsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawBatchCallsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBatchCallsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        call_name: str,
        agent_id: str,
        recipients: typing.Sequence[OutboundCallRecipient],
        scheduled_time_unix: typing.Optional[int] = OMIT,
        agent_phone_number_id: typing.Optional[str] = OMIT,
        whatsapp_params: typing.Optional[BatchCallWhatsAppParams] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        branch_id: typing.Optional[str] = OMIT,
        environment: typing.Optional[str] = OMIT,
        telephony_call_config: typing.Optional[TelephonyCallConfig] = OMIT,
        target_concurrency_limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchCallResponse:
        """
        Submit a batch call request to schedule calls for multiple recipients.

        Parameters
        ----------
        call_name : str

        agent_id : str

        recipients : typing.Sequence[OutboundCallRecipient]

        scheduled_time_unix : typing.Optional[int]

        agent_phone_number_id : typing.Optional[str]

        whatsapp_params : typing.Optional[BatchCallWhatsAppParams]

        timezone : typing.Optional[str]

        branch_id : typing.Optional[str]

        environment : typing.Optional[str]

        telephony_call_config : typing.Optional[TelephonyCallConfig]

        target_concurrency_limit : typing.Optional[int]
            Maximum number of simultaneous calls for this batch. When set, dispatch is governed by this limit rather than workspace/agent capacity percentages.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCallResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs, OutboundCallRecipient

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.batch_calls.create(
            call_name="call_name",
            agent_id="agent_id",
            recipients=[OutboundCallRecipient()],
        )
        """
        _response = self._raw_client.create(
            call_name=call_name,
            agent_id=agent_id,
            recipients=recipients,
            scheduled_time_unix=scheduled_time_unix,
            agent_phone_number_id=agent_phone_number_id,
            whatsapp_params=whatsapp_params,
            timezone=timezone,
            branch_id=branch_id,
            environment=environment,
            telephony_call_config=telephony_call_config,
            target_concurrency_limit=target_concurrency_limit,
            request_options=request_options,
        )
        return _response.data

    def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        last_doc: typing.Optional[str] = None,
        agent_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkspaceBatchCallsResponse:
        """
        Get all batch calls for the current workspace.

        Parameters
        ----------
        limit : typing.Optional[int]

        last_doc : typing.Optional[str]

        agent_id : typing.Optional[str]
            Filter batch calls to a single agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkspaceBatchCallsResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.batch_calls.list(
            limit=1,
            last_doc="last_doc",
            agent_id="agent_id",
        )
        """
        _response = self._raw_client.list(
            limit=limit, last_doc=last_doc, agent_id=agent_id, request_options=request_options
        )
        return _response.data

    def get(
        self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BatchCallDetailedResponse:
        """
        Get detailed information about a batch call including all recipients.

        Parameters
        ----------
        batch_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCallDetailedResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.batch_calls.get(
            batch_id="batch_id",
        )
        """
        _response = self._raw_client.get(batch_id, request_options=request_options)
        return _response.data

    def delete(self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Permanently delete a batch call and all recipient records. Conversations remain in history.

        Parameters
        ----------
        batch_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.batch_calls.delete(
            batch_id="batch_id",
        )
        """
        _response = self._raw_client.delete(batch_id, request_options=request_options)
        return _response.data

    def cancel(self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> BatchCallResponse:
        """
        Cancel a running batch call and set all recipients to cancelled status.

        Parameters
        ----------
        batch_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCallResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.batch_calls.cancel(
            batch_id="batch_id",
        )
        """
        _response = self._raw_client.cancel(batch_id, request_options=request_options)
        return _response.data

    def retry(self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> BatchCallResponse:
        """
        Retry a batch call, calling failed and no-response recipients again.

        Parameters
        ----------
        batch_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCallResponse
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.batch_calls.retry(
            batch_id="batch_id",
        )
        """
        _response = self._raw_client.retry(batch_id, request_options=request_options)
        return _response.data


class AsyncBatchCallsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawBatchCallsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawBatchCallsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBatchCallsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        call_name: str,
        agent_id: str,
        recipients: typing.Sequence[OutboundCallRecipient],
        scheduled_time_unix: typing.Optional[int] = OMIT,
        agent_phone_number_id: typing.Optional[str] = OMIT,
        whatsapp_params: typing.Optional[BatchCallWhatsAppParams] = OMIT,
        timezone: typing.Optional[str] = OMIT,
        branch_id: typing.Optional[str] = OMIT,
        environment: typing.Optional[str] = OMIT,
        telephony_call_config: typing.Optional[TelephonyCallConfig] = OMIT,
        target_concurrency_limit: typing.Optional[int] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> BatchCallResponse:
        """
        Submit a batch call request to schedule calls for multiple recipients.

        Parameters
        ----------
        call_name : str

        agent_id : str

        recipients : typing.Sequence[OutboundCallRecipient]

        scheduled_time_unix : typing.Optional[int]

        agent_phone_number_id : typing.Optional[str]

        whatsapp_params : typing.Optional[BatchCallWhatsAppParams]

        timezone : typing.Optional[str]

        branch_id : typing.Optional[str]

        environment : typing.Optional[str]

        telephony_call_config : typing.Optional[TelephonyCallConfig]

        target_concurrency_limit : typing.Optional[int]
            Maximum number of simultaneous calls for this batch. When set, dispatch is governed by this limit rather than workspace/agent capacity percentages.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCallResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs, OutboundCallRecipient

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.batch_calls.create(
                call_name="call_name",
                agent_id="agent_id",
                recipients=[OutboundCallRecipient()],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            call_name=call_name,
            agent_id=agent_id,
            recipients=recipients,
            scheduled_time_unix=scheduled_time_unix,
            agent_phone_number_id=agent_phone_number_id,
            whatsapp_params=whatsapp_params,
            timezone=timezone,
            branch_id=branch_id,
            environment=environment,
            telephony_call_config=telephony_call_config,
            target_concurrency_limit=target_concurrency_limit,
            request_options=request_options,
        )
        return _response.data

    async def list(
        self,
        *,
        limit: typing.Optional[int] = None,
        last_doc: typing.Optional[str] = None,
        agent_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> WorkspaceBatchCallsResponse:
        """
        Get all batch calls for the current workspace.

        Parameters
        ----------
        limit : typing.Optional[int]

        last_doc : typing.Optional[str]

        agent_id : typing.Optional[str]
            Filter batch calls to a single agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        WorkspaceBatchCallsResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.batch_calls.list(
                limit=1,
                last_doc="last_doc",
                agent_id="agent_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            limit=limit, last_doc=last_doc, agent_id=agent_id, request_options=request_options
        )
        return _response.data

    async def get(
        self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BatchCallDetailedResponse:
        """
        Get detailed information about a batch call including all recipients.

        Parameters
        ----------
        batch_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCallDetailedResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.batch_calls.get(
                batch_id="batch_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(batch_id, request_options=request_options)
        return _response.data

    async def delete(self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Permanently delete a batch call and all recipient records. Conversations remain in history.

        Parameters
        ----------
        batch_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.batch_calls.delete(
                batch_id="batch_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(batch_id, request_options=request_options)
        return _response.data

    async def cancel(
        self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BatchCallResponse:
        """
        Cancel a running batch call and set all recipients to cancelled status.

        Parameters
        ----------
        batch_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCallResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.batch_calls.cancel(
                batch_id="batch_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cancel(batch_id, request_options=request_options)
        return _response.data

    async def retry(
        self, batch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BatchCallResponse:
        """
        Retry a batch call, calling failed and no-response recipients again.

        Parameters
        ----------
        batch_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BatchCallResponse
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.batch_calls.retry(
                batch_id="batch_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retry(batch_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .alerting_monitor_config import AlertingMonitorConfig
from .alerting_webhook_notifier import AlertingWebhookNotifier


class AlertingSettings(UncheckedBaseModel):
    """
    Alerting configuration used at both per-agent and per-workspace level.

    All fields are optional overrides; the cascade resolver fills in defaults
    when they are unset. Notifiers stack and dedupe (by URL) across the
    workspace and agent layers rather than overriding each other.

    Cascade order for per-monitor threshold and auto-resolve: agent → workspace →
    system default.
    """

    monitor_configs: typing.Optional[typing.Dict[str, AlertingMonitorConfig]] = pydantic.Field(default=None)
    """
    Alerting configuration keyed by monitor name.
    """

    auto_resolve_after_inactive_minutes: typing.Optional[int] = pydantic.Field(default=None)
    """
    How many minutes an alert can stay inactive before it is auto-resolved. Unset values fall through to the next layer.
    """

    notifiers: typing.Optional[typing.List[AlertingWebhookNotifier]] = pydantic.Field(default=None)
    """
    Delivery channels for alert lifecycle notifications. Stacked and deduped by URL with other layers.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
